import React, {
  DetailedHTMLProps,
  InputHTMLAttributes,
  useEffect,
  useMemo,
} from 'react';
import { TextInput } from 'react-native';

export interface StringInputProps
  extends Omit<
    DetailedHTMLProps<InputHTMLAttributes<HTMLInputElement>, HTMLInputElement>,
    'ref'
  > {
  'data-type'?: string;
  multiline?: boolean;
  rows?: number;
}

export const StringInput = React.forwardRef<HTMLInputElement, StringInputProps>(
  function StringInput(
    {
      'data-type': dataType,
      onChange,
      onBlur,
      onFocus,
      value,
      maxLength,
      multiline,
      rows,
      disabled,
      ...props
    }: StringInputProps,
    ref
  ) {
    const [currentValue, setValue] = React.useState(value);

    const keyboardType = useMemo(() => {
      switch (dataType) {
        case 'afm':
          return 'number-pad';
        case 'phone_number':
          return 'phone-pad';
        case 'mobile_phone':
          return 'phone-pad';
        case 'postal_code':
          return 'number-pad';
        case 'int':
          return 'numeric';
        case 'email':
          return 'email-address';
        default:
          return 'default';
      }
    }, [dataType]);

    useEffect(() => {
      if (currentValue !== value) {
        handleChange();
      }
    }, [currentValue]);
    useEffect(() => {
      if (currentValue !== value) {
        setValue(value);
      }
    }, [value]);

    const handleChange = () => {
      if (onChange) {
        const syntheticEvent = {
          target: {
            value: currentValue,
          },
        } as React.ChangeEvent<HTMLInputElement>;
        onChange(syntheticEvent);
      }
    };

    const handleBlur = () => {
      if (onBlur) {
        const syntheticEvent = {
          target: {
            value: currentValue,
          },
        } as React.FocusEvent<HTMLInputElement>;
        onBlur(syntheticEvent);
      }
    };

    const handleFocus = () => {
      if (onFocus) {
        const syntheticEvent = {
          target: {
            value: currentValue,
          },
        } as React.FocusEvent<HTMLInputElement>;
        onFocus(syntheticEvent);
      }
    };
    return (
      <TextInput
        ref={ref}
        onChangeText={setValue}
        onBlur={handleBlur}
        onFocus={handleFocus}
        value={currentValue}
        multiline={multiline}
        rows={rows}
        editable={!disabled}
        maxLength={Number(maxLength) || undefined}
        keyboardType={keyboardType}
        {...(props as any)}
      />
    );
  }
);
