import React, { useMemo } from 'react';
import { Text, View, ViewProps } from 'react-native';
import { BaseProps as CoreBaseProps } from '@digigov/react-core/Base/index.web';
import { mapping } from '@digigov/react-core/Base/mapping.native';
import { basePropGenerator } from '@digigov/react-core/Base/utils';
import { wrap } from '@digigov/react-core/Base/wrap.native';

export type BaseProps<C extends React.ElementType> = CoreBaseProps<C> &
  ViewProps;
export type BaseComponent = <C extends React.ElementType = 'span'>(
  props: BaseProps<C>
) => React.ReactNode | null;

const BooleanMapper = {
  true: true,
  false: false,
  undefined: false,
  null: false,
  '': false,
};

const textHtmlTags = ['h1', 'h2', 'h3', 'h4', 'h5', 'p', 'span'];

export const Base: BaseComponent = React.forwardRef(function Base<
  C extends React.ElementType = 'span',
>(
  {
    as,
    display,
    children,
    margin,
    marginTop,
    marginBottom,
    marginLeft,
    marginRight,
    m,
    mt,
    mb,
    ml,
    mr,
    mx,
    my,
    padding,
    paddingTop,
    paddingBottom,
    paddingLeft,
    paddingRight,
    p,
    pt,
    pb,
    pr,
    pl,
    px,
    py,
    printHidden,
    printVisible,
    hidden,
    xsUpHidden,
    smUpHidden,
    mdUpHidden,
    lgUpHidden,
    xlUpHidden,
    xsHidden,
    smHidden,
    mdHidden,
    lgHidden,
    xlHidden,
    className,
    'aria-expanded': ariaExpanded,
    ...props
  }: BaseProps<C>,
  ref: React.ComponentPropsWithRef<C>['ref']
) {
  const classNames = className?.split(' ') || [];
  let applyBasePropsToWrapper = true;

  const basePropToClassName = basePropGenerator({
    display,
    margin,
    marginTop,
    marginBottom,
    marginLeft,
    marginRight,
    m,
    mt,
    mb,
    ml,
    mr,
    mx,
    my,
    padding,
    paddingTop,
    paddingBottom,
    paddingLeft,
    paddingRight,
    p,
    pt,
    pb,
    pr,
    pl,
    px,
    py,
    printHidden,
    printVisible,
    hidden,
    xsUpHidden,
    smUpHidden,
    mdUpHidden,
    lgUpHidden,
    xlUpHidden,
    xsHidden,
    smHidden,
    mdHidden,
    lgHidden,
    xlHidden,
  });

  if (typeof children === 'string') {
    children = [children];
  }
  if (Array.isArray(children)) {
    const textClassNames = classNames
      ?.map((className) => `${className}__text`)
      .join(' ');
    children = children.map((child, index) => {
      if (Array.isArray(child)) {
        return child.map((child, index) => {
          if (typeof child === 'string') {
            return (
              <Text key={index} className={textClassNames}>
                {child}
              </Text>
            );
          }
          return child;
        });
      } else if (typeof child === 'string') {
        if (as && textHtmlTags.includes(as as string)) {
          applyBasePropsToWrapper = false;
        }
        return (
          <Text
            key={index}
            className={
              applyBasePropsToWrapper
                ? textClassNames
                : `${textClassNames} ${basePropToClassName}`
            }
            {...(props.onClick && {
              onPress: props.onClick,
            })}
          >
            {child}
          </Text>
        );
      }
      return child;
    });
  }
  const Component = useMemo(() => {
    const C = mapping[as as keyof JSX.IntrinsicElements] || View;
    if (
      classNames.find((className) =>
        className.match(/__wrapper|__before|__after$/)
      )
    ) {
      return wrap(C);
    }
    return C;
  }, [className]);

  Object.keys(props).forEach((k) => {
    if (k.startsWith('aria')) {
      if (BooleanMapper[props[k as keyof typeof props]] !== undefined) {
        props[k as keyof typeof props] =
          BooleanMapper[props[k as keyof typeof props]];
      }
    }
  });
  return (
    <Component
      {...props}
      ref={ref}
      aria-expanded={
        (ariaExpanded as unknown as string) === 'false'
          ? false
          : ariaExpanded === true
            ? true
            : undefined
      }
      className={
        applyBasePropsToWrapper
          ? `${basePropToClassName} ${className}`
          : className
      }
    >
      {children}
    </Component>
  );
});

export default Base;
