import React from 'react';
import clsx from 'clsx';
import Base, { BaseProps } from '@digigov/react-core/Base';
import { ChevronIcon } from '@digigov/react-icons';
export interface DropdownButtonProps extends BaseProps<'summary'> {
  /**
   * color is optional.
   * 'primary' color gives DropdownButton the primary green color.
   * 'secondary' color gives DropdownButton the secondary gray color.
   * 'warning' color gives DropdownButton the warning red color.
   * The default value is 'primary'.
   * @value 'primary'
   * @value 'secondary'
   * @value 'warning'
   * @default 'primary'
   */
  color?: 'primary' | 'secondary' | 'warning';
  /**
   * variant is optional.
   * variant prop changes DropdownButton to button or link.
   * @value 'link'
   * @value 'button'
   * @default 'button'
   */
  variant?: 'link' | 'button';
  /**
   * arrow is optional.
   * arrow prop adds checkIcon component inside DropdownButton.
   * arrow prop is false by default
   * @value true
   * @value false
   * @default false
   */
  arrow?: boolean;
  /**
   * underline is optional.
   * Use this prop when you want to use Dropdown button as \<th\> header in sorted table.
   * @value true
   * @value false
   */
  underline?: boolean;
}
/**
 * DropdownButton it should be inside the Dropdown and it is used for opening or closing dropdown.
 */
export const DropdownButton = React.forwardRef<
  HTMLElement,
  DropdownButtonProps
>(function DropdownButton(
  {
    className,
    children,
    color = 'primary',
    variant = 'button',
    arrow = false,
    tabIndex = 0,
    underline,
    ...props
  },
  ref
) {
  return (
    <Base
      as="summary"
      ref={ref}
      tabIndex={tabIndex}
      className={clsx(className, {
        'ds-dropdown__button': true,
        'ds-link': variant === 'link',
        'ds-btn': variant === 'button',
        'ds-dropdown__button--chevron': arrow,
        'ds-dropdown__button--underline': underline,
        'ds-btn-primary': variant === 'button' && color === 'primary',
        'ds-btn-secondary': variant === 'button' && color === 'secondary',
        'ds-btn-warning': variant === 'button' && color === 'warning',
      })}
      {...props}
    >
      {children}
      {arrow && <ChevronIcon direction="down" />}
    </Base>
  );
});

export default DropdownButton;
