import React from 'react';
import clsx from 'clsx';
import Base, { BaseProps } from '@digigov/react-core/Base';

type GridValues = 'auto' | 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 | 9 | 10 | 11 | 12;
type SpacingValues = 1 | 2 | 3 | 4 | 5 | 6 | 8 | 10 | 11 | 12;
interface SpacingMediaValues {
  xs?: SpacingValues;
  sm?: SpacingValues;
  md?: SpacingValues;
  lg?: SpacingValues;
  xl?: SpacingValues;
}
export interface GridProps extends BaseProps<'div' | 'aside' | 'main'> {
  /**
   * If true, the component will have the flex container behavior. You should be wrapping items with a container.
   * @value true
   * @value false
   * @default false
   */
  container?: boolean;
  /**
   * @value true
   * @value false
   * @default true
   */
  item?: boolean;
  /**
   * @value true
   * @value false
   */
  inline?: boolean;
  /**
   * Defines the space between the type item components. It can only be used on a type container component.
   * @value 1
   * @value 2
   * @value 4
   * @value 6
   * @value 8
   * @value 10
   * @value 12
   * @value object
   * @default undefined
   */
  spacing?: SpacingValues | SpacingMediaValues;

  /**
   * If a number, it sets the number of columns the grid item uses.
   * It can't be greater than the total number of columns of the container (12).
   * If 'auto', the grid item's width matches its content.
   * If it's not set, the prop is ignored.
   * The value is applied for all the screen sizes with the lowest priority.
   */
  xs?: GridValues;
  /**
   * If a number, it sets the number of columns the grid item uses.
   * It can't be greater than the total number of columns of the container (12).
   * If 'auto', the grid item's width matches its content.
   * If it's not set, the prop is ignored.
   * The value is applied for the sm breakpoint and wider screens if not overridden.
   */
  sm?: GridValues;
  /**
   * If a number, it sets the number of columns the grid item uses.
   * It can't be greater than the total number of columns of the container (12).
   * If 'auto', the grid item's width matches its content.
   * If it's not set, the prop is ignored.
   * The value is applied for the md breakpoint and wider screens if not overridden.
   */
  md?: GridValues;
  /**
   * If a number, it sets the number of columns the grid item uses.
   * It can't be greater than the total number of columns of the container (12).
   * If 'auto', the grid item's width matches its content.
   * If it's not set, the prop is ignored.
   * The value is applied for the lg breakpoint and wider screens if not overridden.
   */
  lg?: GridValues;
  /**
   * If a number, it sets the number of columns the grid item uses.
   * It can't be greater than the total number of columns of the container (12).
   * If 'auto', the grid item's width matches its content.
   * If it's not set, the prop is ignored.
   * The value is applied for the xl breakpoint and wider screens.
   */
  xl?: GridValues;
}
/**
 * Grid is used to arrange other components in complex grid configurations of rows and columns.
 */
export const Grid = React.forwardRef<HTMLElement, GridProps>(function Grid(
  {
    className,
    children,
    container = false,
    item = true,
    spacing,
    inline,
    xs,
    sm,
    md,
    lg,
    xl,
    as,
    ...props
  },
  ref
) {
  return (
    <Base
      as={as || 'div'}
      ref={ref}
      className={clsx(className, {
        'ds-grid': container,
        'ds-grid-inline': container && inline,
        [`ds-gap-${spacing}`]: container && typeof spacing === 'number',
        [`ds-gap-xs-${(spacing as SpacingMediaValues)?.xs}`]:
          container && (spacing as SpacingMediaValues)?.xs !== undefined,
        [`ds-gap-sm-${(spacing as SpacingMediaValues)?.sm}`]:
          container && (spacing as SpacingMediaValues)?.sm !== undefined,
        [`ds-gap-md-${(spacing as SpacingMediaValues)?.md}`]:
          container && (spacing as SpacingMediaValues)?.md !== undefined,
        [`ds-gap-lg-${(spacing as SpacingMediaValues)?.lg}`]:
          container && (spacing as SpacingMediaValues)?.lg !== undefined,
        [`ds-gap-xl-${(spacing as SpacingMediaValues)?.xl}`]:
          container && (spacing as SpacingMediaValues)?.xl !== undefined,
        [`xs:ds-grid__col-span-${xs}`]: item && xs !== undefined,
        [`sm:ds-grid__col-span-${sm}`]: item && sm !== undefined,
        [`md:ds-grid__col-span-${md}`]: item && md !== undefined,
        [`lg:ds-grid__col-span-${lg}`]: item && lg !== undefined,
        [`xl:ds-grid__col-span-${xl}`]: item && xl !== undefined,
      })}
      {...props}
    >
      {children}
    </Base>
  );
});

export default Grid;
