import React from 'react';
import clsx from 'clsx';
import Typography, { TypographyProps } from '@digigov/react-core/Typography';

export interface HeadingProps extends TypographyProps {
  /**
   * size is optional. 'xl' is the default value.
   * Use 'xl' when this heading is the main page title.
   * @value 'xl'
   * @value 'lg'
   * @value 'md'
   * @value 'sm'
   * @default 'xl'
   */
  size?: 'xl' | 'lg' | 'md' | 'sm' | 'xs';
  /**
   * element is optional. 'h1' is the default value.
   * Use 'h1' when this heading is the main page title.
   */
  element?: HeadingHTMLElements;
  /**
   * breakWords is optional. false is the default value.
   * Use true when you want to break words.
   */
  breakWords?: boolean;
}
export type HeadingHTMLElements = 'h1' | 'h2' | 'h3' | 'h4' | 'h5';

const getComponentBySize = (
  size: HeadingProps['size']
): HeadingHTMLElements => {
  switch (size) {
    case 'xl':
      return 'h1';
    case 'lg':
      return 'h2';
    case 'md':
      return 'h3';
    case 'sm':
      return 'h4';
    case 'xs':
      return 'h5';
    default:
      return 'h1';
  }
};

/**
 * Heading component is used to place a title in a the page.
 * Heading component must be wrapped inside Main component.
 */
export const Heading = React.forwardRef<HTMLHeadingElement, HeadingProps>(
  function Heading(
    { size = 'xl', element, breakWords, className, children, ...props },
    ref
  ) {
    const Component = element ? element : getComponentBySize(size);
    return (
      <Typography
        as={Component}
        ref={ref}
        className={clsx(className, {
          [`ds-heading-${size}`]: size !== undefined,
          'ds-heading--break-words': breakWords,
        })}
        {...props}
      >
        {children}
      </Typography>
    );
  }
);

export default Heading;
