import React from 'react';
import clsx from 'clsx';
import Base, { BaseProps } from '@digigov/react-core/Base';
import CloseButton from '@digigov/react-core/CloseButton';

export interface NavProps extends BaseProps<'nav'> {
  /**
   * open is optional.
   * use this prop to control the navigation's visibility when it's a drawer.
   * @value true
   * @value false
   * @default false
   */
  open?: boolean;
  /**
   * border is optional.
   * border prop contains border-bottom styling.
   * @value true
   * @value false
   * @default true
   */
  border?: boolean;

  /**
   * drawer is optional.
   * drawer prop allows element to be positioned relative to the viewport.
   * Use this prop only if you want to implement responsive drawer navigation and you have a button to toggle the drawer open and closed.
   * @value 'never' the navigation will never be drawer
   * @value 'smDown' the navigation will be drawer on small screens only
   * @value 'mdDown' the navigation will be drawer on medium screens and below
   * @value 'lgDown' the navigation will be drawer on large screens and below
   * @value 'always' the navigation will be drawer on all screen sizes
   * @default 'never'
   */
  drawer?: 'never' | 'smDown' | 'mdDown' | 'lgDown' | 'always';

  /**
   * onClose makes a CloseButton component visible when the Nav is drawer.
   * Use this prop to close the drawer when close button is clicked, passing a callback function.
   * Make sure that the drawer prop is not 'never'.
   */
  onClose?: (e: React.MouseEvent<HTMLButtonElement, MouseEvent>) => void;

  /**
   * closeButtonLabel is optional.
   * Use this prop to add text beside the close icon. For example: 'Κλείσιμο'.
   */
  closeButtonLabel?: string;
}
/**
 * Nav is mainly used under the Header.
 * This is a wrapper for the NavList component.
 */
export const Nav = React.forwardRef<HTMLElement, NavProps>(function Nav(
  {
    className,
    open,
    drawer = 'never',
    onClose,
    closeButtonLabel,
    border = true,
    children,
    ...props
  },
  ref
) {
  return (
    <Base
      as="nav"
      ref={ref}
      className={clsx(className, {
        'ds-nav': true,
        'ds-nav--border': border,
        'ds-nav--hidden': drawer != 'never' && !open,
        'ds-nav--drawer-smDown': drawer === 'smDown',
        'ds-nav--drawer-mdDown': drawer === 'mdDown',
        'ds-nav--drawer-lgDown': drawer === 'lgDown',
        'ds-nav--drawer': drawer === 'always',
      })}
      {...props}
    >
      {onClose && drawer !== 'never' && (
        <CloseButton
          smUpHidden={drawer === 'smDown'}
          mdUpHidden={drawer === 'mdDown'}
          lgUpHidden={drawer === 'lgDown'}
          onClick={(e) => onClose(e)}
          label={closeButtonLabel}
        />
      )}
      {children}
    </Base>
  );
});

export default Nav;
