import React from 'react';
import clsx from 'clsx';
import Base, { BaseProps } from '@digigov/react-core/Base';

export interface TableProps extends BaseProps<'table'> {
  /**
   * headerVariant is optional.
   * The `headerVariant` prop is used to either give blue background color to th tags
   * or a light grey background.
   * Default value is undefined which results to an opaque background color.
   * @value 'dark'
   * @value 'light'
   */
  headerVariant?: 'dark' | 'light';
  /**
   * dense is optional.
   * The dense prop is used to make font size smaller and give
   * extra padding.
   * @value true
   * @value false
   * @default false
   */
  dense?: boolean;
  /**
   * verticalBorders is optional.
   * The verticalBorders prop gives extra vertical lines to the table.
   * @value true
   * @value false
   * @default false
   */
  verticalBorders?: boolean;
  /**
   * stacked is optional.
   * The stacked prop gives responsiveness to the table. If used, make sure to add "data-label" to TableDataCell components.
   * Default value is never.
   * @value 'always'
   * @value 'sm'
   * @value 'md'
   * @value 'never'
   * @default 'never'
   */
  stacked?: 'always' | 'sm' | 'md' | 'never';
  /**
   * variant is optional.
   * @value 'default'
   * @value 'zebra' is used to make zebra table lines. If you have 'zebra' variant, color prop to TableRow component adds nothing.
   * @default 'default'
   */
  variant?: 'zebra' | 'default';
}
/**
 * Table component is used to wrap the sub components of a table
 */
export const Table = React.forwardRef<HTMLTableElement, TableProps>(
  function Table(
    {
      headerVariant,
      variant,
      dense = false,
      verticalBorders = false,
      stacked = 'never',
      className,
      children,
      ...props
    },
    ref
  ) {
    return (
      <Base
        as="table"
        ref={ref}
        className={clsx(className, {
          'ds-table': true,
          'ds-table--stacked-always': stacked === 'always',
          'ds-table--stacked-sm': stacked === 'sm',
          'ds-table--stacked-md': stacked == 'md',
          'ds-table--dark': headerVariant === 'dark',
          'ds-table--light': headerVariant === 'light',
          'ds-table--dense': dense,
          'ds-table--zebra': variant === 'zebra',
          'ds-table--vertical-lines': verticalBorders,
        })}
        {...props}
      >
        {children}
      </Base>
    );
  }
);

export default Table;
