import React from 'react';

type WithClassName<TProps> = TProps & { className?: string };
type PropFactory<TProps> = (props: WithClassName<TProps>) => any;

/**
 * A HOC that transforms the props of the wrapped component before passing them through
 *
 * @param Component The component to wrap
 * @param [propFactory=(props) => props] The function that transforms the props
 * @returns The wrapped component that accepts the transformed props
 *
 * @example
 * type MyComponentProps = { name: string; };
 * const MyComponent = (props: MyComponentProps) => <p>Hi {props.name}</p>;
 *
 * const MyNativeComponent = extend(MyComponent, (props) => ({ name: props.name.toUpperCase(), ...props }));
 *
 * type AdditionalProps = { lastName: string; };
 * const MyExpandedNativeComponent = extend<MyComponentProps, AdditionalProps>(MyNativeComponent, (props) => ({ name: `${props.name} ${props.lastName}`, ...props }));
 */
export function extend<TProps, TAdditionalProps>(
  Component: React.ComponentType<TProps>,
  propFactory: PropFactory<TProps & TAdditionalProps> = (props) => props
) {
  const ExtendedComponent = React.forwardRef(function ExtendedComponent(
    props: WithClassName<TProps & TAdditionalProps>,
    ref
  ) {
    const transformedProps = propFactory(props);

    return <Component ref={ref} {...transformedProps} />;
  });
  return ExtendedComponent;
}
export default extend;
