import React from 'react';
import clsx from 'clsx';
import { basePropGenerator } from '@digigov/react-core/Base/utils';
interface AsProp<C extends React.ElementType> {
  as?: C;
}

type PropsToOmit<C extends React.ElementType, P> = keyof (AsProp<C> & P);

// This is the first reusable type utility we built
type PolymorphicComponentProp<
  C extends React.ElementType,
  Props = Record<string, unknown>,
> = React.PropsWithChildren<Props & AsProp<C>> &
  Omit<React.ComponentPropsWithoutRef<C>, PropsToOmit<C, Props>>;

// This is a new type utitlity with ref!
type PolymorphicComponentPropWithRef<
  C extends React.ElementType,
  Props = Record<string, unknown>,
> = PolymorphicComponentProp<C, Props> & { ref?: any };

// This is the type for the "ref" only
type PolymorphicRef<C extends React.ElementType> =
  React.ComponentPropsWithRef<C>['ref'];

/**
 * This is the updated component props using PolymorphicComponentPropWithRef
 */
export type BaseProps<C extends React.ElementType> =
  PolymorphicComponentPropWithRef<
    C,
    {
      /** margin is optional. It has not a default value. */
      margin?: SpacingValues | SpacingMediaValues;
      /** marginTop is optional. It has not a default value. */
      marginTop?: SpacingValues | SpacingMediaValues;
      /** marginBottom is optional. It has not a default value. */
      marginBottom?: SpacingValues | SpacingMediaValues;
      /** marginLeft is optional. It has not a default value. */
      marginLeft?: SpacingValues | SpacingMediaValues;
      /** marginRight is optional. It has not a default value. */
      marginRight?: SpacingValues | SpacingMediaValues;
      /** Alternative for margin. Margin is optional. It has not a default value. */
      m?: SpacingValues | SpacingMediaValues;
      /** Alternative for marginTop. MarginTop is optional. It has not a default value. */
      mt?: SpacingValues | SpacingMediaValues;
      /** Alternative for marginBottom. MarginBottom is optional. It has not a default value. */
      mb?: SpacingValues | SpacingMediaValues;
      /** Alternative for marginLeft. MarginLeft is optional. It has not a default value. */
      ml?: SpacingValues | SpacingMediaValues;
      /** Alternative for marginRight. MarginRight is optional. It has not a default value. */
      mr?: SpacingValues | SpacingMediaValues;
      /** Alternative for marginX. MarginX is optional. It has not a default value. */
      mx?: SpacingValues | SpacingMediaValues;
      /** Alternative for marginY. MarginY is optional. It has not a default value. */
      my?: SpacingValues | SpacingMediaValues;
      /** padding is optional. It has not a default value. */
      padding?: SpacingValues | SpacingMediaValues;
      /** paddingTop is optional. It has not a default value. */
      paddingTop?: SpacingValues | SpacingMediaValues;
      /** paddingBottom is optional. It has not a default value. */
      paddingBottom?: SpacingValues | SpacingMediaValues;
      /** paddingLeft is optional. It has not a default value. */
      paddingLeft?: SpacingValues | SpacingMediaValues;
      /** paddingRight is optional. It has not a default value. */
      paddingRight?: SpacingValues | SpacingMediaValues;
      /** Alternative for padding. Padding is optional. It has not a default value. */
      p?: SpacingValues | SpacingMediaValues;
      /** Alternative for paddingTop. PaddingTop is optional. It has not a default value. */
      pt?: SpacingValues | SpacingMediaValues;
      /** Alternative for paddingBottom. PaddingBottom is optional. It has not a default value. */
      pb?: SpacingValues | SpacingMediaValues;
      /** Alternative for paddingLeft. PaddingLeft is optional. It has not a default value. */
      pl?: SpacingValues | SpacingMediaValues;
      /** Alternative for paddingRight. PaddingRight is optional. It has not a default value. */
      pr?: SpacingValues | SpacingMediaValues;
      /** Alternative for paddingX. PaddingX is optional. It has not a default value. */
      px?: SpacingValues | SpacingMediaValues;
      /** Alternative for paddingY. PaddingY is optional. It has not a default value. */
      py?: SpacingValues | SpacingMediaValues;
      /**  printHidden is optional. Default value is false. When true, the component is hidden at print. */
      printHidden?: boolean;
      /**  printVisible is optional. When block, the component is displayed as block. When inline, the component is displayed as inline.*/
      printVisible?: 'block' | 'inline';
      /** Components will be hidden at all screen sizes. */
      hidden?: boolean;
      /** Components will be hidden from 'xs' screen size and up. */
      xsUpHidden?: boolean;
      /** Components will be hidden from 'sm' screen size and up. */
      smUpHidden?: boolean;
      /** Components will be hidden from 'md' screen size and up. */
      mdUpHidden?: boolean;
      /** Components will be hidden from 'lg' screen size and up. */
      lgUpHidden?: boolean;
      /** Components will be hidden from 'xl' screen size and up. */
      xlUpHidden?: boolean;
      /** Components will be hidden between xs and sm screen size. */
      xsHidden?: boolean;
      /** Components will be hidden between sm and md screen size. */
      smHidden?: boolean;
      /** Components will be hidden between md and lg screen size. */
      mdHidden?: boolean;
      /** Components will be hidden between lg and xl screen size. */
      lgHidden?: boolean;
      /** Components will be hidden between xl and 2xl screen size. */
      xlHidden?: boolean;
      /**  display is optional. It has not a default value. */
      display?: 'flex' | 'block' | 'none';
    }
  >;

export type SpacingValues =
  | 0
  | 0.5
  | 1
  | 1.5
  | 2
  | 2.5
  | 3
  | 3.5
  | 4
  | 5
  | 6
  | 7
  | 8
  | 9
  | 10
  | 11
  | 12
  | 14
  | 16
  | 20
  | 24
  | 28
  | 32
  | 36
  | 40
  | 44
  | 48
  | 52
  | 56
  | 60
  | 64
  | 72
  | 80
  | 96;

export interface SpacingMediaValues {
  xs?: SpacingValues;
  sm?: SpacingValues;
  md?: SpacingValues;
  lg?: SpacingValues;
  xl?: SpacingValues;
}

/**
 * This is the type used in the type annotation for the component
 */
type BaseComponent = <C extends React.ElementType = 'span'>(
  props: BaseProps<C>
) => React.ReactNode | null;

export const Base: BaseComponent = React.forwardRef(function Base<
  C extends React.ElementType = 'span',
>(
  {
    as,
    display,
    margin,
    marginTop,
    marginBottom,
    marginLeft,
    marginRight,
    m,
    mt,
    mb,
    ml,
    mr,
    mx,
    my,
    padding,
    paddingTop,
    paddingBottom,
    paddingLeft,
    paddingRight,
    p,
    pt,
    pb,
    pr,
    pl,
    px,
    py,
    printHidden,
    printVisible,
    hidden,
    xsUpHidden,
    smUpHidden,
    mdUpHidden,
    lgUpHidden,
    xlUpHidden,
    xsHidden,
    smHidden,
    mdHidden,
    lgHidden,
    xlHidden,
    children,
    className,
    ...props
  }: BaseProps<C>,
  ref?: PolymorphicRef<C>
) {
  const Component = as || 'span';

  const basePropToClassName = basePropGenerator({
    display,
    margin,
    marginTop,
    marginBottom,
    marginLeft,
    marginRight,
    m,
    mt,
    mb,
    ml,
    mr,
    mx,
    my,
    padding,
    paddingTop,
    paddingBottom,
    paddingLeft,
    paddingRight,
    p,
    pt,
    pb,
    pr,
    pl,
    px,
    py,
    printHidden,
    printVisible,
    hidden,
    xsUpHidden,
    smUpHidden,
    mdUpHidden,
    lgUpHidden,
    xlUpHidden,
    xsHidden,
    smHidden,
    mdHidden,
    lgHidden,
    xlHidden,
  });

  return (
    <Component
      className={clsx(basePropToClassName, className)}
      ref={ref}
      {...props}
    >
      {children}
    </Component>
  );
});

export default Base;
