import React, { useContext, useMemo, useState } from 'react';
import { View } from 'react-native';

type WrapperContextType = (cls: string) => void;

const WrapperContext = React.createContext<WrapperContextType>(
  (cls: string) => {
    return cls;
  }
);

const Wrapper = React.memo(function Wrapper({
  children,
}: {
  children?: React.ReactNode;
}) {
  const [className, setClassName] = useState('');
  return (
    <WrapperContext.Provider value={setClassName}>
      <View className={className} focusable={false}>
        {children}
      </View>
    </WrapperContext.Provider>
  );
});
Wrapper.displayName = 'Wrapper';

interface WrappedComponentProps {
  className?: string;
  onBlur?: (e: unknown) => void;
  onFocus?: (e: unknown) => void;
}

export const wrap = (Component: React.ElementType) => {
  const WrappedComponent = React.forwardRef(function InnerComponent(
    { className, onBlur, onFocus, ...props }: WrappedComponentProps,
    ref
  ) {
    const setClassName = useContext(WrapperContext);
    const wrapperClassNames: string[] = [];
    const beforeClassNames: string[] = [];
    const afterClassNames: string[] = [];
    const focusedWrapperClassNames: string[] = [];
    const classNames = useMemo(
      () =>
        (className?.split(' ') || []).filter((cls) => {
          if (cls.endsWith('__wrapper')) {
            wrapperClassNames.push(cls);
            focusedWrapperClassNames.push(`${cls}--focus`);
            return false;
          } else if (cls.endsWith('__before')) {
            beforeClassNames.push(cls);
            return false;
          } else if (cls.endsWith('__after')) {
            afterClassNames.push(cls);
            return false;
          }
          return true;
        }),
      [className]
    );
    return (
      <>
        {beforeClassNames.length ? (
          <View focusable={false} className={beforeClassNames.join(' ')} />
        ) : (
          ''
        )}
        <Component
          ref={ref}
          onFocus={(e: unknown) => {
            setClassName(focusedWrapperClassNames.join(' '));
            if (onFocus) onFocus(e);
          }}
          onBlur={(e: unknown) => {
            setClassName(wrapperClassNames.join(' '));
            if (onBlur) onBlur(e);
          }}
          className={classNames.join(' ')}
          {...props}
        />
        {afterClassNames.length ? (
          <View focusable={false} className={afterClassNames.join(' ')} />
        ) : (
          ''
        )}
      </>
    );
  });
  return React.forwardRef(function Wrapped(props, ref) {
    const children = useMemo(
      () => <WrappedComponent {...props} ref={ref} />,
      []
    );
    return <Wrapper>{children}</Wrapper>;
  });
};
