import React from 'react';
import clsx from 'clsx';
import Base, { BaseProps } from '@digigov/react-core/Base';

export interface DrawerProps extends BaseProps<'div'> {
  /**
   * open is optional.
   * This declares if the drawer is open at small screens.
   * @value true
   * @value false
   */
  open?: boolean;

  /**
   * direction is optional.
   * Add the direction of the drawer, when this is open at mobile view.
   * The default value is 'left'.
   * @value 'right'
   * @value 'left'
   * @default 'left'
   */
  direction?: 'right' | 'left';

  /**
   * upRelative is optional.
   * This declares if the drawer is visible at what screens in a relative position.
   * If not relative, it will be a drawer.
   * The default value is never.
   * @value 'sm' The drawer will be visible and relative at small screens and up
   * @value 'md' The drawer will be visible and relative at medium screens and up
   * @value 'lg' The drawer will be visible and relative at large screens and up
   * @value 'never' The drawer will never have relative position in the page
   * @default 'never'
   */
  upRelative?: 'sm' | 'md' | 'lg' | 'never';

  /**
   * bgColor is optional.
   * This defines the background color of the drawer.
   * @value 'base'
   * @value 'secondary'
   * @default 'base'
   */
  bgColor?: 'base' | 'secondary';
}
/**
 * Drawer can be used to provide access to destinations and app functionality.
 * It can either be permanently on-screen or controlled by a navigation menu icon.
 */
export const Drawer = React.forwardRef<HTMLDivElement, DrawerProps>(
  function Drawer(
    {
      open,
      direction = 'left',
      upRelative = 'never',
      bgColor,
      className,
      children,
      ...props
    },
    ref
  ) {
    return (
      <Base
        as="div"
        ref={ref}
        className={clsx({
          'ds-drawer-container': true,
          'ds-drawer-container--relative-sm': upRelative === 'sm',
          'ds-drawer-container--relative-md': upRelative === 'md',
          'ds-drawer-container--relative-lg': upRelative === 'lg',
          'ds-drawer-container--closed': !open,
        })}
      >
        <Base
          as="div"
          className={clsx(className, {
            'ds-drawer': true,
            'ds-drawer--relative-sm': upRelative === 'sm',
            'ds-drawer--relative-md': upRelative === 'md',
            'ds-drawer--relative-lg': upRelative === 'lg',
            'ds-drawer--closed': !open,
            'ds-drawer--right': direction === 'right',
            'ds-drawer--left': direction === 'left',
            'ds-drawer--bg-secondary': bgColor === 'secondary',
          })}
          {...props}
        >
          {children}
        </Base>
      </Base>
    );
  }
);

export default Drawer;
