import React from 'react';
import clsx from 'clsx';
import Base, { BaseProps } from '@digigov/react-core/Base';

export interface FilterContainerProps extends BaseProps<'div'> {
  /**
   * Use open prop to open the filters when at mobile.
   * The default value is false.
   * @value true
   * @value false
   * @default false
   */
  open?: boolean;

  /**
   * Use border prop to add a gray border at the filters section.
   * The default value is true.
   * @value true
   * @value false
   * @default true
   */
  border?: boolean;

  /**
   * positionRelative is optional.
   * Use positionRelative prop to make the filter section to be visible and have position relative in the page.
   * If not relative, the filter section will be a drawer.
   * The default value is md.
   * @value 'always' The filter section will always visible
   * @value 'sm' The filter section will be visible at small screens and up
   * @value 'md' The filter section will be visible at medium screens and up
   * @value 'lg' The filter section will be visible at large screens and up
   * @value 'never' The filter section will never have relative position in the page
   * @default 'md'
   */
  positionRelative?: 'always' | 'sm' | 'md' | 'lg' | 'never';
}

/**
 * Use FilterContainer component to wrap the filter area of the page
 * When the Filters are being used inside a Drawer component, you don't have to
 * use the FilterContainer component
 */
export const FilterContainer = React.forwardRef<
  HTMLDivElement,
  FilterContainerProps
>(function FilterContainer(
  {
    className,
    border = true,
    open = false,
    positionRelative = 'md',
    children,
    ...props
  },
  ref
) {
  return (
    <Base
      as="div"
      ref={ref}
      className={clsx({
        'ds-filter-container': true,
        'ds-filter-container--position-relative-always':
          positionRelative === 'always',
        'ds-filter-container--position-relative-sm': positionRelative === 'sm',
        'ds-filter-container--position-relative-md': positionRelative === 'md',
        'ds-filter-container--position-relative-lg': positionRelative === 'lg',
        'ds-filter-container--closed': !open,
      })}
    >
      <Base
        as="div"
        className={clsx(className, {
          'ds-filter': true,
          'ds-filter--border ': border,
          'ds-filter--position-relative-always': positionRelative === 'always',
          'ds-filter--position-relative-sm': positionRelative === 'sm',
          'ds-filter--position-relative-md': positionRelative === 'md',
          'ds-filter--position-relative-lg': positionRelative === 'lg',
        })}
        {...props}
      >
        {children}
      </Base>
    </Base>
  );
});

export default FilterContainer;
