import React from 'react';
import clsx from 'clsx';
import Base, { BaseProps } from '@digigov/react-core/Base';

export interface SkeletonProps extends BaseProps<'span'> {
  /**
   * variant prop determines the style of the skeleton element.
   * @value 'circular', it renders a circular skeleton.
   * @value 'rectangular', it renders a rectangular skeleton.
   * @value 'text', it renders a text skeleton.
   * @value 'button', it renders a button skeleton.
   * @default 'text'
   */
  variant?: 'circular' | 'rectangular' | 'text' | 'button';

  /**
   * width prop determines the width of the skeleton element.
   * It accepts number in px or string with valid CSS units (e.g. '24px', '1em', '100%').
   * For better responsive experience, avoid using px and use percentage instead.
   * @default undefined
   */
  width?: number | string;

  /**
   * height prop determines the height of the skeleton element.
   * It accepts number in px or string with valid CSS units (e.g. '24px', '1em', '100%').
   * @default undefined
   */
  height?: number | string;

  /**
   * fontSize is optional. 'sm' is the default value.
   * Use 'xl' when this Skeleton is for a main page title.
   * The fontSize works only for variant 'text'.
   * You don't need fontSize if you wrap Skeleton in a typography component.
   * @value 'xl'
   * @value 'lg'
   * @value 'md'
   * @value 'sm'
   * @value 'xs'
   * @default 'sm'
   */
  fontSize?: 'xl' | 'lg' | 'md' | 'sm' | 'xs';
  /**
   * animation prop determines if the skeleton should animate or not.
   * @default true
   */
  animation?: boolean;
}
/**
 * Use Skeleton component while a page is loading.
 */
export const Skeleton = React.forwardRef<HTMLSpanElement, SkeletonProps>(
  function Skeleton(
    {
      width,
      height,
      fontSize,
      variant = 'text',
      animation = true,
      className,
      children,
      style,
      ...props
    },
    ref
  ) {
    return (
      <Base
        as="span"
        ref={ref}
        className={clsx(className, {
          'ds-skeleton': true,
          [`ds-skeleton--font-${fontSize}`]:
            fontSize !== undefined && variant === 'text',
          [`ds-skeleton--${variant}`]:
            !!variant && (!children || typeof children === 'string'),
          'ds-skeleton--animate': !!animation,
          'ds-skeleton--width-fit-content': !!children && !width,
        })}
        {...props}
        style={{
          width:
            typeof children === 'string'
              ? Math.ceil(children.length * 0.5) + 'em'
              : width,
          maxWidth: width ? '100%' : undefined,
          height: height,
          ...style,
        }}
      >
        {variant === 'button' && (
          <Base
            as="span"
            className={clsx({
              'ds-skeleton__line': true,
              'ds-skeleton__line--size-default':
                width === undefined && height === undefined,
            })}
          ></Base>
        )}
        {typeof children != 'string' && children}
      </Base>
    );
  }
);

export default Skeleton;
