import React from 'react';
import clsx from 'clsx';
import Base, { BaseProps } from '@digigov/react-core/Base';

export interface StackProps extends BaseProps<'div'> {
  /**
   * direction is optional. Set to control the direction of flex items.
   */
  direction?: 'row' | 'row-reverse' | 'column' | 'column-reverse';
  /**
   * justifyContent is optional. Set to control how items are positioned along Stack's main axis .
   */
  justifyContent?:
    | 'flex-start'
    | 'flex-end'
    | 'center'
    | 'space-between'
    | 'space-around'
    | 'space-evenly';
  /**
   * alignItems is optional. Set to control how items are positioned along Stack's cross axis.
   */
  alignItems?: 'stretch' | 'flex-start' | 'flex-end' | 'center' | 'baseline';
  /**
   * alignContent is optional. Set to control how rows are positioned in multi-row Stack container.
   */
  alignContent?:
    | 'flex-start'
    | 'flex-end'
    | 'center'
    | 'space-between'
    | 'space-around'
    | 'space-evenly';
  /**
   * spacing is optional. Set to the number you want the space to be.
   */
  spacing?: SpacingValues | SpacingMediaValues;
  /**
   * flexWrap is optional. The default value is 'wrap'.
   */
  flexWrap?: 'wrap' | 'nowrap';
}

type SpacingValues = 1 | 2 | 3 | 4 | 5 | 6 | 8 | 10 | 11 | 12;
interface SpacingMediaValues {
  xs?: SpacingValues;
  sm?: SpacingValues;
  md?: SpacingValues;
  lg?: SpacingValues;
  xl?: SpacingValues;
}
/**
 * ...
 */
export const Stack = React.forwardRef<HTMLDivElement, StackProps>(
  function Stack(
    {
      direction = 'column',
      justifyContent,
      alignItems,
      alignContent,
      spacing,
      flexWrap,
      className,
      children,
      ...props
    },
    ref
  ) {
    return (
      <Base
        as="div"
        ref={ref}
        className={clsx(className, {
          'ds-stack': true,
          [`ds-stack--${direction}`]: direction,
          [`ds-stack--justify-${justifyContent}`]: justifyContent,
          [`ds-stack--align-${alignItems}`]: alignItems,
          [`ds-stack--content-${alignContent}`]: alignContent,
          'ds-stack--nowrap': flexWrap === 'nowrap',
          [`ds-gap-${spacing}`]: spacing !== undefined,
          [`ds-gap-xs-${(spacing as SpacingMediaValues)?.xs}`]:
            (spacing as SpacingMediaValues)?.xs !== undefined,
          [`ds-gap-sm-${(spacing as SpacingMediaValues)?.sm}`]:
            (spacing as SpacingMediaValues)?.sm !== undefined,
          [`ds-gap-md-${(spacing as SpacingMediaValues)?.md}`]:
            (spacing as SpacingMediaValues)?.md !== undefined,
          [`ds-gap-lg-${(spacing as SpacingMediaValues)?.lg}`]:
            (spacing as SpacingMediaValues)?.lg !== undefined,
          [`ds-gap-xl-${(spacing as SpacingMediaValues)?.xl}`]:
            (spacing as SpacingMediaValues)?.xl !== undefined,
        })}
        {...props}
      >
        {children}
      </Base>
    );
  }
);

export default Stack;
