import React from 'react';
import clsx from 'clsx';
import Base, { BaseProps } from '@digigov/react-core/Base';

export interface TextInputProps extends BaseProps<'input'> {
  /**
   * maxWidth is optional.
   * @value '2-char';
   * @value '3-char';
   * @value '4-char';
   * @value '5-char';
   * @value '10-char';
   * @value '20-char';
   */
  maxWidth?: '2-char' | '3-char' | '4-char' | '5-char' | '10-char' | '20-char';

  /**
   * width is optional.
   * Use width prop to define a custom width in your component.
   * @value '25%';
   * @value '33.3%';
   * @value '50%';
   * @value '66.6%';
   * @value '75%';
   * @value '100%';
   * @value 'full';
   */
  width?: '25%' | '33.3%' | '50%' | '66.6%' | '75%' | '100%' | 'full';

  /**
   * error is optional. The default value is false.
   * Use this prop when there is an error at the input.
   * @value true
   * @value false
   */
  error?: boolean;
  /**
   * dense is optional.
   * @value true TextInput will be dense.
   * @value false
   * @default false
   */
  dense?: boolean;
}
/**
 * TextInput component when you need to let users enter text that’s no
 * longer than a single line, such as their name or phone number.
 */
export const TextInput = React.forwardRef<HTMLInputElement, TextInputProps>(
  function TextInput(
    {
      type = 'text',
      maxWidth,
      width,
      error,
      dense,
      className,
      children,
      ...props
    },
    ref
  ) {
    return (
      <Base
        as="input"
        type={type}
        ref={ref}
        className={clsx(className, {
          'ds-input': true,
          'ds-input--error': error,
          'ds-input--dense': dense,
          [`ds-input--width-${maxWidth}`]: maxWidth !== undefined,
          'ds-!-width-one-quarter': width === '25%',
          'ds-!-width-one-third': width === '33.3%',
          'ds-!-width-one-half': width === '50%',
          'ds-!-width-two-thirds': width === '66.6%',
          'ds-!-width-three-quarters': width === '75%',
        })}
        {...props}
      >
        {children}
      </Base>
    );
  }
);

export default TextInput;
